#!/usr/bin/env bash
usage_title="Mosaic Wrapper"
usage_description=""

function set_usage {
  usage_title="$1"
  usage_description="$2"
}

function usage {
  local error="${1:-}"
  local self
  self="$(basename "$0")"

  if [ "${error}" ]; then
    error=$'\n\n'"${error}"$'\n'
  fi

  cat >&2 <<EOF
${usage_title}

Usage:
  ${self} [-v] check <directory>
  ${self} [-v] build <directory>
  ${self} [-v] show <directory>
  ${self} [-v] deploy <directory>
  ${self} [-v] destroy <directory>
  ${self} -h

Commands:
  check   Validate all known resources.
  build   Build and push Docker container.
  show    Show proposed changes for all known resources.
  deploy  Deploy all known resources.
  destroy Destroy all known resources.

Options:
  -h  Show this screen.
  -v  Verbose output.

${usage_description}${error}
EOF
  exit 64
}

verbose=

function if_verbose {
  if [ -n "${verbose}" ]; then
    printf '%s\n' "$(tr '\n' ' ' <<<"$*")"
  fi
  "$@"
}

function parse_args {
  while getopts ":hv" OPT; do
    case "${OPT}" in
      h)
        # Show help.
        usage
        ;;
      v)
        # Verbose output.
        verbose='true'
        ;;
      \?)
        # Invalid argument.
        usage "Invalid option: ${OPT}"
        ;;
    esac
  done

  shift $((OPTIND -1))

  # Ignore shellcheck warnings for returning action & directory, since they're
  # used by the sourcing script.
  case "$1" in
    check)
      # shellcheck disable=SC2034
      action=check
      ;;
    build)
      # shellcheck disable=SC2034
      action=build
      ;;
    show)
      # shellcheck disable=SC2034
      action=show
      ;;
    deploy)
      # shellcheck disable=SC2034
      action=deploy

      # Pause deployments for MITRE Attack testing
      if [[ $ENV_CATEGORY = "prod" ]]; then
         if [[ (( $(date --utc +%s) > $(date --utc -d "2021-10-07T19:00:00" +%s) )) &&
            (( $(date --utc +%s) < $(date --utc -d "2021-10-15T14:00:00" +%s) )) ]]; then

          echo "#####################################################"
          echo "# Deployments are paused for MITRE Attack & Testing #"
          echo "# Pause will end Friday Morning on October 15th     #"
          echo "# Please contact Lavie Tobey with any questions     #"
          echo "#####################################################"
          exit 1;

        fi
      fi

      mosaic-notify-slack || true
      ;;
    destroy)
      # shellcheck disable=SC2034
      action=destroy
      ;;
    *)
      usage "Invalid argument: $1"
      ;;
  esac

  # shellcheck disable=SC2034
  directory="${2:-.}"
}

# Needs to be run in each script context to unset any ENV AWS creds which take precendence
# and allow the creds in ~/.aws/credentials to be used
function unset_AWS_env_creds {
  unset AWS_ACCESS_KEY_ID
  unset AWS_SECRET_ACCESS_KEY
  unset AWS_SESSION_TOKEN
}
