#!/usr/bin/env bash
set -euo pipefail
IFS=$'\n'

# For when script is called directly
if [ "${ENABLE_HELM3-false}" = "true" ]; then
  export PATH=/usr/local/bin/helm3/:$PATH
fi

# shellcheck source=src/bin/mosaic-common.bash
source "$(dirname "${BASH_SOURCE[0]}")/mosaic-common.bash"
set_usage "Mosaic helm Wrapper" "
Find all helm charts under the given directory and run the given command
against them.

Any directory containing a \"helm-upgrade.bash\" or \"Chart.yaml\" file is
considered a helm chart.
"

function helm_directories {
  local under="${1:-.}"
  find -L "${under}" \( -name '.*' \! -name '.' \) -prune -o \
    -name 'helm-upgrade.bash' -o -name 'Chart.yaml' \
    -type f -exec dirname \{\} \; | \
    sort -u
}

function helm_check {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    if [ -e helm-diff.bash ]; then
      if_verbose shellcheck helm-diff.bash
    fi
    if [ -e helm-upgrade.bash ]; then
      if_verbose grep 'helm.*upgrade.*--install' helm-upgrade.bash
      if_verbose grep 'helm.*upgrade.*--version' helm-upgrade.bash
      if_verbose shellcheck helm-upgrade.bash
    fi
    if [ -e helm-delete.bash ]; then
      if_verbose shellcheck helm-delete.bash
    fi
    if [ -e Chart.yaml ]; then
      if_verbose helm lint
    fi
  )
}

function helm_show {
  local directory="$1"

  (
    if_verbose cd "${directory}"
    if [ -e helm-diff.bash ]; then
      if_verbose ./helm-diff.bash
    elif [ -e Chart.yaml ]; then
      if_verbose helm diff
    fi
  )
}

function helm_deploy {
  local directory="$1"
  local release
  release="$(basename "${directory}")"

  (
    helm version --short
    if_verbose cd "${directory}"
    helm init --client-only
    if [ -e helm-upgrade.bash ]; then
      if_verbose ./helm-upgrade.bash
    elif [ -e Chart.yaml ]; then
      if_verbose helm repo update
      if_verbose helm upgrade --namespace "${TILLER_NAMESPACE}" --wait --install "${release}" .
    fi
    if_verbose helm test --namespace "${TILLER_NAMESPACE}" --cleanup "${release}"
  )
}

function helm_destroy {
  local directory="$1"
  local release
  release="$(basename "${directory}")"

  (
    if_verbose cd "${directory}"
    helm init --client-only
    if [ -e helm-delete.bash ]; then
      if_verbose ./helm-delete.bash
    elif [ -e Chart.yaml ]; then
      if_verbose helm delete --namespace "${TILLER_NAMESPACE}" --purge "${release}"
    fi
  )
}

mosaic-configure.bash
unset_AWS_env_creds
parse_args "$@"

case "${action}" in
  check)
    for d in $(helm_directories "${directory}"); do
      helm_check "${d}"
    done
    ;;
  show)
    for d in $(helm_directories "${directory}"); do
      helm_show "${d}"
    done
    ;;
  deploy)
    for d in $(helm_directories "${directory}"); do
      helm_deploy "${d}"
    done
    ;;
  destroy)
    for d in $(helm_directories "${directory}"); do
      helm_destroy "${d}"
    done
    ;;
  *)
    usage
    ;;
esac
